\name{pwAdjMatrices}
\alias{pwAdjMatrices}
\title{PathWave adjacency matrices builder}
\description{
Builds a list of and -if wanted prints- adjacency matrices for a list of bipartite graphs.
}
\usage{
pwAdjMatrices(reactionList, compoundList, printMatrices = FALSE, matrixType = "adjacency")
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{reactionList}{List of reactions. See details.}
  \item{compoundList}{List of compounds. See details.}
  \item{printMatrices}{Built matrices are printed in the working directory (default FALSE). See details.}
  \item{matrixType}{Printed matrices could be of type "adjacency" or "distance" (default "adjacency").}
}
\details{
  \item{reactionList}{List of reactions as returned by pwKEGGxml() -> reactions. List of graph IDs. For every graph ID a vector of the reaction IDs is stored.}
  \item{compoundList}{List of compoundss as returned by pwKEGGxml() -> compounds. List of graph IDs. For every graph ID various list of reaction IDs are stored. Every reaction ID list has a vector of compounds connected to the specific reaction.}
  \item{printMatrices}{File names for printed matrices are built by concatenating pathway ID with ending ".matrix". First two numbers in file are number of nodes and length of shortest path.}
}
\value{
  A list is returned:
  \item{pathway ID}{List of different pathways.}
  
  For every pathway ID a list is stored:
  \item{nNodes}{Integer, number of nodes (reactions) in graph.}
  \item{iMax}{Integer, length of longest shortest path between all pairs of reactions in graph.}
  \item{M}{Adjacency matrix for the pathway ID.}
  \item{dist}{Distance matrix for the pathway ID.}
}
\author{Gunnar Schramm}
\seealso{See Also as \code{\link{pwKEGGxml}},\code{\link{pwLPFiles}}}
\examples{
#read in actual KEGG xml files
kegg.ftp="ftp://ftp.genome.jp/pub/kegg/xml/organisms/hsa/"
keggXml=pwKEGGxml(kegg.ftp)

#build adjacency matrices
adjacencyMatrices=pwAdjMatrices(reactionList=keggXml$reactions,compoundList=keggXml$compounds,printMatrices=FALSE)
}
\keyword{manip}
\keyword{graphs}
\keyword{optimize}
