\name{pwKEGGxml}
\alias{pwKEGGxml}
\title{PathWave parser for KEGG XML files}
\description{
Reads and parses KEGG xml files. Extracts gene IDs, reaction IDs, compound IDs and internal KEGG IDs for every pathway.
}
\usage{
pwKEGGxml(url = "ftp://ftp.genome.jp/pub/kegg/xml/organisms/hsa/")
}
\arguments{
  \item{url}{Url or directory containing the xml files that should be parsed (default "ftp://ftp.genome.jp/pub/kegg/xml/organisms/hsa/"). See details.}
}
\details{
  \item{url}{Variable url can either be an url or a directory. The function searches for xml files in the given url.}  
  IMPORTANT: reaction IDs are extracted for each pathway. To avoid mismatching similar reaction IDs of different pathways, pathway ID and reaction ID are concatenated, e.g. hsa00251:R00256 . 
}
\value{
  A List is returned:
  \item{ids}{Internal KEGG IDs. Contains a list of pathway IDs. For every pathway ID a vector is stored with the internal KEGG IDs for every reaction ID.}
  \item{genes}{Contains a list of pathway IDs. For every pathway ID a vector is stored with the entrez gene IDs for every reaction ID. Reaction IDs can occur more than once if more than one gene can be mapped on it.}
  \item{reactions}{Contains a list of pathway IDs. For every pathway ID a vector is stored with reaction IDs (concatenated with the pathway ID).}
  \item{compounds}{Contains a list of pathway IDs. For every pathway ID a list of the pathways reaction IDs is stored. These lists contain the compounds the specific reaction is connected with.}
}
\references{KEGG: Kanehisa, M., Araki, M., Goto, S., Hattori, M., Hirakawa, M., Itoh, M., Katayama, T., Kawashima, S., Okuda, S., Tokimatsu, T., and Yamanishi, Y.; KEGG for linking genomes to life and the environment. Nucleic Acids Res. 36, D480-D484 (2008).}
\author{Gunnar Schramm}
\seealso{See Also as \code{\link{pwAdjMatrices}}}
\examples{
kegg.ftp="ftp://ftp.genome.jp/pub/kegg/xml/organisms/hsa/"
keggXml=pwKEGGxml(kegg.ftp)
}
\keyword{manip}
\keyword{graphs}
\keyword{optimize}
